#pragma once

#include <stdint.h>
#include <avr/io.h>


namespace Devices::Mcu
{
    struct Timer2
    {
        enum class Mode
        {
            Normal,
            PhaseCorrectPwm1,
            ClearOnCompareMatch,
            FastPwm3,
            PhaseCorrectPwm5,
            FastPwm7
        };


        enum class ClockSource
        {
            Disabled = 0x00,
            ClkDiv1 = 0x01,
            ClkDiv8 = 0x02,
            ClkDiv32 = 0x03,
            ClkDiv64 = 0x04,
            ClkDiv128 = 0x05,
            ClkDiv256 = 0x06,
            ClkDiv1024 = 0x07
        };
        
        static void SetMode(Mode const mode)
        {
            switch (mode)
            {
                case Mode::Normal:
                    TCCR2B &= ~_BV(WGM22);
                    TCCR2A &= ~_BV(WGM21) & ~_BV(WGM20);
                    break;
                    
                case Mode::PhaseCorrectPwm1:
                    TCCR2B &= ~_BV(WGM22);
                    TCCR2A &= ~_BV(WGM21);
                    TCCR2A |= _BV(WGM20);
                    break;
                    
                case Mode::ClearOnCompareMatch:
                    TCCR2B &= ~_BV(WGM22);
                    TCCR2A |= _BV(WGM21);
                    TCCR2A &= ~_BV(WGM20);
                    break;
                    
                case Mode::FastPwm3:
                    TCCR2B &= ~_BV(WGM22);
                    TCCR2A |= _BV(WGM21) | _BV(WGM20);;
                    break;

                case Mode::PhaseCorrectPwm5:
                    TCCR2B |= _BV(WGM22);
                    TCCR2A &= ~_BV(WGM21);
                    TCCR2A |= _BV(WGM20);
                    break;                    

                case Mode::FastPwm7:
                    TCCR2B |= _BV(WGM22);
                    TCCR2A |= _BV(WGM21) | _BV(WGM20);
                    break;                    
            }
        }


        static void SetClockSource(ClockSource const clockSource)
        {
            TCCR2B &= ~_BV(CS22) & ~_BV(CS21) & ~_BV(CS20);
            TCCR2B |= (uint8_t) clockSource;
        }


        static void SetInterrupts(bool overflow, bool outputCompareA, bool outputCompareB)
        {
            if (overflow)
            {
                TIMSK2 |= _BV(TOIE2);
            }
            else
            {
                TIMSK2 &= ~_BV(TOIE2);
            }

            if (outputCompareA)
            {
                TIMSK2 |= _BV(OCIE2A);
            }
            else
            {
                TIMSK2 &= ~_BV(OCIE2A);
            }

            if (outputCompareB)
            {
                TIMSK2 |= _BV(OCIE2B);
            }
            else
            {
                TIMSK2 &= ~_BV(OCIE2B);
            }
        }


        static void SetValue(uint8_t const value)
        {
            TCNT2 = value;
        }


        static bool IsOverflown()
        {
            return (TIFR2 & _BV(TOV2)) != 0;
        }
    };
}